@@ -1,5 +1,5 @@
 /*
- *  guage.c -- implements the guage dialog
+ *  guage.c -- implements the gauge dialog
  *
  *  AUTHOR: Marc Ewing, Red Hat Software
  *
@@ -25,7 +25,7 @@
  */
 
 /*
- * Display a guage, or progress meter.  Starts at percent% and
+ * Display a gauge, or progress meter.  Starts at percent% and
  * reads stdin.  If stdin is not XXX, then it is interpreted as
  * a percentage, and the display is updated accordingly.  Otherwise
  * the next line is the percentage, and subsequent lines up to
@@ -34,7 +34,7 @@
  * larger than the height and width specified.
  */
 int
-dialog_guage (const char *title, const char *prompt, int height,
+dialog_gauge (const char *title, const char *prompt, int height,
 		int width, int percent)
 {
     int i, x, y;
@@ -60,6 +60,8 @@ dialog_guage (const char *title, const char *prompt, int height,
 	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
 	guint input;
 	GIOChannel *giochannel;
+
+	GList *labellist;
 	
 	label_autowrap(vbox, prompt, width);
 	gtk_progress_set_percentage( GTK_PROGRESS( p ), ( (gfloat) percent ) / 100);
@@ -71,6 +73,13 @@ dialog_guage (const char *title, const char *prompt, int height,
 	giochannel = g_io_channel_unix_new (0);
 	input = g_io_add_watch(giochannel, G_IO_IN, callback_progress_bar, (gpointer) p);
 
+	if(GTK_IS_ACCESSIBLE(gtk_widget_get_accessible(p))) {
+		labellist = gtk_container_get_children(GTK_CONTAINER(vbox));
+		add_atk_namedesc(p, _("Gdialog progressbar"), _("Indicates the percentage specified at the standard input"));
+		add_atk_relation(GTK_WIDGET(labellist->data), p, ATK_RELATION_LABEL_FOR);
+		add_atk_relation(p, GTK_WIDGET(labellist->data), ATK_RELATION_LABELLED_BY);
+	}
+
 	gtk_widget_show_all(w);
 	gtk_main();
 	exit( 0 );
